@@ -39,6 +39,9 @@ use std::num::NonZeroU32;
 use std::ops::Index;
 use std::ops::IndexMut;
 
+use bitvec::vec::BitVec;
+use controlled_option::Niche;
+
 use crate::utils::cmp_option;
 use crate::utils::equals_option;
 
@@ -60,7 +63,7 @@ pub struct Handle<T> {
 }
 
 impl<T> Handle<T> {
-    fn new(index: NonZeroU32) -> Handle<T> {
+    pub(crate) fn new(index: NonZeroU32) -> Handle<T> {
         Handle {
             index,
             _phantom: PhantomData,
@@ -78,6 +81,30 @@ impl<T> Handle<T> {
     }
 }
 
+impl<T> Niche for Handle<T> {
+    type Output = u32;
+
+    #[inline]
+    fn none() -> Self::Output {
+        0
+    }
+
+    #[inline]
+    fn is_none(value: &Self::Output) -> bool {
+        *value == 0
+    }
+
+    #[inline]
+    fn into_some(value: Self) -> Self::Output {
+        value.index.get()
+    }
+
+    #[inline]
+    fn from_some(value: Self::Output) -> Self {
+        Self::new(unsafe { NonZeroU32::new_unchecked(value) })
+    }
+}
+
 // Normally we would #[derive] all of these traits, but the auto-derived implementations all
 // require that T implement the trait as well.  We don't store any real instances of T inside of
 // Handle, so our implementations do _not_ require that.
@@ -231,15 +258,24 @@ impl<T> Arena<T> {
 /// [`Arena`]: struct.Arena.html
 /// [`get`]: #method.get
 pub struct SupplementalArena<H, T> {
-    items: Vec<T>,
+    items: Vec<MaybeUninit<T>>,
     _phantom: PhantomData<H>,
 }
 
+impl<H, T> Drop for SupplementalArena<H, T> {
+    fn drop(&mut self) {
+        unsafe {
+            let items = std::mem::transmute::<_, &mut [T]>(&mut self.items[1..]) as *mut [T];
+            items.drop_in_place();
+        }
+    }
+}
+
 impl<H, T> SupplementalArena<H, T> {
     /// Creates a new, empty supplemental arena.
     pub fn new() -> SupplementalArena<H, T> {
         SupplementalArena {
-            items: Vec::new(),
+            items: vec![MaybeUninit::uninit()],
             _phantom: PhantomData,
         }
     }
@@ -247,20 +283,37 @@ impl<H, T> SupplementalArena<H, T> {
     /// Creates a new, empty supplemental arena, preallocating enough space to store supplemental
     /// data for all of the instances that have already been allocated in a (regular) arena.
     pub fn with_capacity(arena: &Arena<H>) -> SupplementalArena<H, T> {
+        let mut items = Vec::with_capacity(arena.items.len());
+        items[0] = MaybeUninit::uninit();
         SupplementalArena {
-            items: Vec::with_capacity(arena.items.len()),
+            items,
             _phantom: PhantomData,
         }
     }
 
     /// Returns the item belonging to a particular handle, if it exists.
     pub fn get(&self, handle: Handle<H>) -> Option<&T> {
-        self.items.get(handle.as_usize() - 1)
+        self.items
+            .get(handle.as_usize())
+            .map(|x| unsafe { &*(x.as_ptr()) })
     }
 
     /// Returns a mutable reference to the item belonging to a particular handle, if it exists.
     pub fn get_mut(&mut self, handle: Handle<H>) -> Option<&mut T> {
-        self.items.get_mut(handle.as_usize() - 1)
+        self.items
+            .get_mut(handle.as_usize())
+            .map(|x| unsafe { &mut *(x.as_mut_ptr()) })
+    }
+
+    /// Returns a pointer to this arena's storage.
+    pub(crate) fn as_ptr(&self) -> *const T {
+        self.items.as_ptr() as *const T
+    }
+
+    /// Returns the number of instances stored in this arena.
+    #[inline(always)]
+    pub fn len(&self) -> usize {
+        self.items.len()
     }
 }
 
@@ -272,10 +325,11 @@ where
     /// (using the type's `Default` implementation) if it doesn't already exist.
     pub fn get_mut_or_default(&mut self, handle: Handle<H>) -> &mut T {
         let index = handle.as_usize();
-        if self.items.len() < index {
-            self.items.resize_with(index, || T::default());
+        if self.items.len() <= index {
+            self.items
+                .resize_with(index + 1, || MaybeUninit::new(T::default()));
         }
-        unsafe { self.items.get_unchecked_mut(index - 1) }
+        unsafe { std::mem::transmute(&mut self.items[handle.as_usize()]) }
     }
 }
 
@@ -288,7 +342,7 @@ impl<H, T> Default for SupplementalArena<H, T> {
 impl<H, T> Index<Handle<H>> for SupplementalArena<H, T> {
     type Output = T;
     fn index(&self, handle: Handle<H>) -> &T {
-        &self.items[handle.as_usize() - 1]
+        unsafe { std::mem::transmute(&self.items[handle.as_usize()]) }
     }
 }
 
@@ -301,6 +355,79 @@ where
     }
 }
 
+//-------------------------------------------------------------------------------------------------
+// Handle sets
+
+/// Contains a set of handles, encoded efficiently using a bit set.
+#[repr(C)]
+pub struct HandleSet<T> {
+    elements: BitVec<bitvec::order::Lsb0, u32>,
+    _phantom: PhantomData<T>,
+}
+
+impl<T> HandleSet<T> {
+    /// Creates a new, empty handle set.
+    pub fn new() -> HandleSet<T> {
+        HandleSet::default()
+    }
+
+    /// Removes all elements from this handle set.
+    pub fn clear(&mut self) {
+        self.elements.clear();
+    }
+
+    /// Returns whether this set contains a particular handle.
+    pub fn contains(&self, handle: Handle<T>) -> bool {
+        let index = handle.as_usize();
+        self.elements.get(index).map(|bit| *bit).unwrap_or(false)
+    }
+
+    /// Adds a handle to this set.
+    pub fn add(&mut self, handle: Handle<T>) {
+        let index = handle.as_usize();
+        if self.elements.len() <= index {
+            self.elements.resize(index + 1, false);
+        }
+        let mut bit = unsafe { self.elements.get_unchecked_mut(index) };
+        *bit = true;
+    }
+
+    /// Removes a handle from this set.
+    pub fn remove(&mut self, handle: Handle<T>) {
+        let index = handle.as_usize();
+        if let Some(mut bit) = self.elements.get_mut(index) {
+            *bit = false;
+        }
+    }
+
+    /// Returns an iterator of all of the handles in this set.
+    pub fn iter(&self) -> impl Iterator<Item = Handle<T>> + '_ {
+        self.elements
+            .iter_ones()
+            .map(|index| Handle::new(unsafe { NonZeroU32::new_unchecked(index as u32) }))
+    }
+
+    /// Returns a pointer to this set's storage.
+    pub(crate) fn as_ptr(&self) -> *const u32 {
+        self.elements.as_raw_ptr()
+    }
+
+    /// Returns the number of instances stored in this arena.
+    #[inline(always)]
+    pub(crate) fn len(&self) -> usize {
+        self.elements.as_raw_slice().len()
+    }
+}
+
+impl<T> Default for HandleSet<T> {
+    fn default() -> HandleSet<T> {
+        HandleSet {
+            elements: BitVec::default(),
+            _phantom: PhantomData,
+        }
+    }
+}
+
 //-------------------------------------------------------------------------------------------------
 // Arena-allocated lists
 
@@ -310,10 +437,12 @@ where
 /// linked list implementation _should_ be cache-friendly, since the individual cells are allocated
 /// out of an arena.
 #[repr(C)]
+#[derive(Niche)]
 pub struct List<T> {
     // The value of this handle will be EMPTY_LIST_HANDLE if the list is empty.  For an
     // Option<List<T>>, the value will be zero (via the Option<NonZero> optimization) if the list
     // is None.
+    #[niche]
     cells: Handle<ListCell<T>>,
 }
 
@@ -464,7 +593,9 @@ impl<T> Copy for List<T> {}
 ///
 /// [`List`]: struct.List.html
 #[repr(C)]
+#[derive(Niche)]
 pub struct ReversibleList<T> {
+    #[niche]
     cells: Handle<ReversibleListCell<T>>,
 }
 
@@ -729,7 +860,9 @@ impl<T> Copy for ReversibleList<T> {}
 ///
 /// [`List`]: struct.List.html
 #[repr(C)]
+#[derive(Niche)]
 pub struct Deque<T> {
+    #[niche]
     list: ReversibleList<T>,
     direction: DequeDirection,
 }
@@ -778,6 +911,11 @@ impl<T> Deque<T> {
         }
     }
 
+    /// Returns whether we have already calculated the reversal of this deque.
+    pub fn have_reversal(&self, arena: &DequeArena<T>) -> bool {
+        self.list.have_reversal(arena)
+    }
+
     fn is_backwards(&self) -> bool {
         matches!(self.direction, DequeDirection::Backwards)
     }
